"""Basic functions for OpenNews."""
import requests as r
from bs4 import BeautifulSoup as bs
import aiohttp


def _fetch(url):
    return r.get(url).text


async def _aio_fetch(url):
    async with aiohttp.ClientSession() as session:
        async with session.get(url) as response:
            return await response.text()


def _parse(html):
    return bs(html, 'html.parser')


def _get_news(_extract, url):
    def get_news():
        html = _fetch(url)
        soup = _parse(html)
        news = _extract(soup)
        return news
    return get_news


def _get_news_async(url, _extract):
    async def get_news_async():
        html = await _aio_fetch(url)
        soup = _parse(html)
        news = _extract(soup)
        return news
    return get_news_async



"""CNBC fetcher for OpenNews."""
from .basic import _get_news, _get_news_async

URL = 'https://www.cnbc.com/'


def _extract(soup):
    news = {
        i.get('href'): i.text
        for i in soup.find_all('a') if (
            i.get('href').count("/") == 6
        )
    }
    return news


get_news = _get_news(URL, _extract)
get_news_async = _get_news_async(URL, _extract)




from . import cnn, fox, msnbc, nbc, washingtonpost, theguardian, nytimes, usatoday, wsj, reuters, cnbc


def get_all_news():
    return {
        **cnn.get_news(),
        **fox.get_news(),
        **msnbc.get_news(),
        **nbc.get_news(),
        **washingtonpost.get_news(),
        **theguardian.get_news(),
        **nytimes.get_news(),
        **usatoday.get_news(),
        **wsj.get_news(),
        **reuters.get_news(),
        **cnbc.get_news(),
    }


async def get_all_news_async():
    return {
        **await cnn.get_news_async(),
        **await fox.get_news_async(),
        **await msnbc.get_news_async(),
        **await nbc.get_news_async(),
        **await washingtonpost.get_news_async(),
        **await theguardian.get_news_async(),
        **await nytimes.get_news_async(),
        **await usatoday.get_news_async(),
        **await wsj.get_news_async(),
        **await reuters.get_news_async(),
        **await cnbc.get_news_async(),
    }





"""Washington Post fetcher for OpenNews."""
from .basic import _get_news, _get_news_async


URL = 'https://www.washingtonpost.com/'


def _extract(soup):
    news = {
        str(i.get('href')): i.find("span").text if i.text is None else i.text
        for i in soup.find_all('a') if (
            i.get("class") != "art-link" and i.get("href").count("/") >= 7
        )
    }
    return news


get_news = _get_news(URL, _extract)
get_news_async = _get_news_async(URL, _extract)




"""NBC fetcher for OpenNews."""
from .basic import _get_news, _get_news_async


URL = 'https://www.nbcnews.com/'


def _extract(soup):
    news = {i.get('href'): i.text for i in soup.find_all('a') if "/news/" in i.get('href') and i.get('href').count("/") >= 5 and len(i.text.split(" ")) > 3}
    return news


get_news = _get_news(URL, _extract)
get_news_async = _get_news_async(URL, _extract)




"""MSNBC fetcher for OpenNews."""
from .basic import _get_news, _get_news_async


URL = 'https://www.msnbc.com/'


def _extract(soup):
    news = {i.get('href'): i.text for i in soup.find_all('a') if i.get('href').count("/") == 5 and i.get("target") != "_blank" and "/watch/" not in i.get('href')}
    return news


get_news = _get_news(URL, _extract)
get_news_async = _get_news_async(URL, _extract)




"""New York Times fetcher for OpenNews."""
from .basic import _get_news, _get_news_async


URL = "https://www.nytimes.com/"


def _extract(soup):
    news = {
        str(i.get('href')): i.text[:100]
        for i in soup.find_all('a') if (
            i.get('href').count('/') >= 7
        )
    }
    return news


get_news = _get_news(URL, _extract)
get_news_async = _get_news_async(URL, _extract)




"""Fox News fetcher for OpenNews."""
from .basic import _get_news, _get_news_async

URL = 'https://www.foxnews.com/'


def _extract(soup):
    news = {
        str(i.get('href')): i.text
        for i in soup.find_all('a') if (
            i.get("aria-label") is None and (i.get("href").count('/') == 4 or "video" in i.get("href")) and not any(map(i.get('href').__contains__, ["fox-nation", "podcast", "sp=watch-live", "/category"])) and "https://" in i.get("href") and i.text
        )
    }
    return news


get_news = _get_news(URL, _extract)
get_news_async = _get_news_async(URL, _extract)




"""USAToday fetcher for OpenNews."""
from .basic import _get_news, _get_news_async


URL = 'https://www.usatoday.com/'


def _extract(soup):
    news = {
        URL+str(i.get('href')): i.text[:100]
        for i in soup.find_all('a') if (
            i.get('href') and i.get('href').startswith("/story")
        )
    }
    return news


get_news = _get_news(URL, _extract)
get_news_async = _get_news_async(URL, _extract)




"""Reuters fetcher for OpenNews."""
from .basic import _get_news, _get_news_async

URL = 'https://www.reuters.com'


def _extract(soup):
    news = {
        URL+str(i.get('href')): i.text[:100]
        for i in soup.find_all('a') if (
            i.get('href').startswith("/") and i.get("href").count("/") >= 3 and len(i.text.split(" ")) > 3
        )
    }
    return news


get_news = _get_news(URL, _extract)
get_news_async = _get_news_async(URL, _extract)




"""CNN fetcher for OpenNews."""
from .basic import _get_news, _get_news_async

URL = 'https://lite.cnn.com/en'


def _extract(soup):
    news = {URL+i.get('href'): i.text for i in soup.find_all('a') if i.get('href').count('/') == 3}
    return news


get_news = _get_news(URL, _extract)
get_news_async = _get_news_async(URL, _extract)




"""The Guardian fetcher for OpenNews."""
from .basic import _get_news, _get_news_async

URL = 'https://www.theguardian.com/international'


def _extract(soup):
    news = {
        str(i.get('href')): i.text
        for i in soup.find_all('a') if (
            i.get("data-link-name") in ["article", "article-link"] or "news |" in i.get("data-link-name", '')
        )
    }
    return news


get_news = _get_news(URL, _extract)
get_news_async = _get_news_async(URL, _extract)

